/*!
 * Link Click tracking for Snowplow v4.6.3 (http://bit.ly/sp-js)
 * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
 * Licensed under BSD-3-Clause
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.snowplowLinkClickTracking = {}));
})(this, (function (exports) { 'use strict';

  /******************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */

  function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
  }

  typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
  };

  /*!
   * Core functionality for Snowplow JavaScript trackers v4.6.3 (http://bit.ly/sp-js)
   * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * Licensed under BSD-3-Clause
   */

  /*
   * Copyright (c) 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * All rights reserved.
   *
   * Redistribution and use in source and binary forms, with or without
   * modification, are permitted provided that the following conditions are met:
   *
   * 1. Redistributions of source code must retain the above copyright notice, this
   *    list of conditions and the following disclaimer.
   *
   * 2. Redistributions in binary form must reproduce the above copyright notice,
   *    this list of conditions and the following disclaimer in the documentation
   *    and/or other materials provided with the distribution.
   *
   * 3. Neither the name of the copyright holder nor the names of its
   *    contributors may be used to endorse or promote products derived from
   *    this software without specific prior written permission.
   *
   * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
   * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   */
  function payloadBuilder() {
      var dict = {}, allJson = [], jsonForProcessing = [], contextEntitiesForProcessing = [];
      var processor;
      var add = function (key, value) {
          if (value != null && value !== '') {
              // null also checks undefined
              dict[key] = value;
          }
      };
      var addDict = function (dict) {
          for (var key in dict) {
              if (Object.prototype.hasOwnProperty.call(dict, key)) {
                  add(key, dict[key]);
              }
          }
      };
      var addJson = function (keyIfEncoded, keyIfNotEncoded, json) {
          if (json && isNonEmptyJson(json)) {
              var jsonWithKeys = { keyIfEncoded: keyIfEncoded, keyIfNotEncoded: keyIfNotEncoded, json: json };
              jsonForProcessing.push(jsonWithKeys);
              allJson.push(jsonWithKeys);
          }
      };
      var addContextEntity = function (entity) {
          contextEntitiesForProcessing.push(entity);
      };
      return {
          add: add,
          addDict: addDict,
          addJson: addJson,
          addContextEntity: addContextEntity,
          getPayload: function () { return dict; },
          getJson: function () { return allJson; },
          withJsonProcessor: function (jsonProcessor) {
              processor = jsonProcessor;
          },
          build: function () {
              processor === null || processor === void 0 ? void 0 : processor(this, jsonForProcessing, contextEntitiesForProcessing);
              return dict;
          },
      };
  }
  /**
   * Is property a non-empty JSON?
   * @param property - Checks if object is non-empty json
   */
  function isNonEmptyJson(property) {
      if (!isJson(property)) {
          return false;
      }
      for (var key in property) {
          if (Object.prototype.hasOwnProperty.call(property, key)) {
              return true;
          }
      }
      return false;
  }
  /**
   * Is property a JSON?
   * @param property - Checks if object is json
   */
  function isJson(property) {
      return (typeof property !== 'undefined' &&
          property !== null &&
          (property.constructor === {}.constructor || property.constructor === [].constructor));
  }

  /*
   * Copyright (c) 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * All rights reserved.
   *
   * Redistribution and use in source and binary forms, with or without
   * modification, are permitted provided that the following conditions are met:
   *
   * 1. Redistributions of source code must retain the above copyright notice, this
   *    list of conditions and the following disclaimer.
   *
   * 2. Redistributions in binary form must reproduce the above copyright notice,
   *    this list of conditions and the following disclaimer in the documentation
   *    and/or other materials provided with the distribution.
   *
   * 3. Neither the name of the copyright holder nor the names of its
   *    contributors may be used to endorse or promote products derived from
   *    this software without specific prior written permission.
   *
   * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
   * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   */
  var label = 'Snowplow: ';
  var LOG_LEVEL;
  (function (LOG_LEVEL) {
      LOG_LEVEL[LOG_LEVEL["none"] = 0] = "none";
      LOG_LEVEL[LOG_LEVEL["error"] = 1] = "error";
      LOG_LEVEL[LOG_LEVEL["warn"] = 2] = "warn";
      LOG_LEVEL[LOG_LEVEL["debug"] = 3] = "debug";
      LOG_LEVEL[LOG_LEVEL["info"] = 4] = "info";
  })(LOG_LEVEL || (LOG_LEVEL = {}));
  var LOG = logger();
  function logger(logLevel) {
      if (logLevel === void 0) { logLevel = LOG_LEVEL.warn; }
      function setLogLevel(level) {
          if (LOG_LEVEL[level]) {
              logLevel = level;
          }
          else {
              logLevel = LOG_LEVEL.warn;
          }
      }
      /**
       * Log errors, with or without error object
       */
      function error(message, error) {
          var extraParams = [];
          for (var _i = 2; _i < arguments.length; _i++) {
              extraParams[_i - 2] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.error && typeof console !== 'undefined') {
              var logMsg = label + message + '\n';
              if (error) {
                  console.error.apply(console, __spreadArray([logMsg + '\n', error], extraParams, false));
              }
              else {
                  console.error.apply(console, __spreadArray([logMsg], extraParams, false));
              }
          }
      }
      /**
       * Log warnings, with or without error object
       */
      function warn(message, error) {
          var extraParams = [];
          for (var _i = 2; _i < arguments.length; _i++) {
              extraParams[_i - 2] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.warn && typeof console !== 'undefined') {
              var logMsg = label + message;
              if (error) {
                  console.warn.apply(console, __spreadArray([logMsg + '\n', error], extraParams, false));
              }
              else {
                  console.warn.apply(console, __spreadArray([logMsg], extraParams, false));
              }
          }
      }
      /**
       * Log debug messages
       */
      function debug(message) {
          var extraParams = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              extraParams[_i - 1] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.debug && typeof console !== 'undefined') {
              console.debug.apply(console, __spreadArray([label + message], extraParams, false));
          }
      }
      /**
       * Log info messages
       */
      function info(message) {
          var extraParams = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              extraParams[_i - 1] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.info && typeof console !== 'undefined') {
              console.info.apply(console, __spreadArray([label + message], extraParams, false));
          }
      }
      return { setLogLevel: setLogLevel, warn: warn, error: error, debug: debug, info: info };
  }
  /**
   * Find dynamic context generating functions and return their results to be merged into the static contexts
   * Combine an array of unchanging contexts with the result of a context-creating function
   *
   * @param dynamicOrStaticContexts - Array of custom context Objects or custom context generating functions
   * @param Parameters - to pass to dynamic context callbacks
   * @returns An array of Self Describing JSON context
   */
  function resolveDynamicContext(dynamicOrStaticContexts) {
      var _a;
      var extraParams = [];
      for (var _i = 1; _i < arguments.length; _i++) {
          extraParams[_i - 1] = arguments[_i];
      }
      return ((_a = dynamicOrStaticContexts === null || dynamicOrStaticContexts === void 0 ? void 0 : dynamicOrStaticContexts.map(function (context) {
          if (typeof context === 'function') {
              try {
                  return context.apply(void 0, extraParams);
              }
              catch (e) {
                  //TODO: provide warning
                  return undefined;
              }
          }
          else {
              return context;
          }
      }).filter(Boolean)) !== null && _a !== void 0 ? _a : []);
  }
  /**
   * Build a self-describing event
   * A custom event type, allowing for an event to be tracked using your own custom schema
   * and a data object which conforms to the supplied schema
   *
   * @param event - Contains the properties and schema location for the event
   * @returns PayloadBuilder to be sent to {@link @snowplow/tracker-core#TrackerCore.track}
   */
  function buildSelfDescribingEvent(event) {
      var _a = event.event, schema = _a.schema, data = _a.data, pb = payloadBuilder();
      var ueJson = {
          schema: 'iglu:com.snowplowanalytics.snowplow/unstruct_event/jsonschema/1-0-0',
          data: { schema: schema, data: data },
      };
      pb.add('e', 'ue');
      pb.addJson('ue_px', 'ue_pr', ueJson);
      return pb;
  }
  /**
   * Build a Link Click Event
   * Used when a user clicks on a link on a webpage, typically an anchor tag `<a>`
   *
   * @param event - Contains the properties for the Link Click event
   * @returns PayloadBuilder to be sent to {@link @snowplow/tracker-core#TrackerCore.track}
   */
  function buildLinkClick(event) {
      var targetUrl = event.targetUrl, elementId = event.elementId, elementClasses = event.elementClasses, elementTarget = event.elementTarget, elementContent = event.elementContent;
      var eventJson = {
          schema: 'iglu:com.snowplowanalytics.snowplow/link_click/jsonschema/1-0-1',
          data: removeEmptyProperties({ targetUrl: targetUrl, elementId: elementId, elementClasses: elementClasses, elementTarget: elementTarget, elementContent: elementContent }),
      };
      return buildSelfDescribingEvent({ event: eventJson });
  }
  /**
   * Returns a copy of a JSON with undefined and null properties removed
   *
   * @param event - JSON object to clean
   * @param exemptFields - Set of fields which should not be removed even if empty
   * @returns A cleaned copy of eventJson
   */
  function removeEmptyProperties(event, exemptFields) {
      if (exemptFields === void 0) { exemptFields = {}; }
      var ret = {};
      for (var k in event) {
          if (exemptFields[k] || (event[k] !== null && typeof event[k] !== 'undefined')) {
              ret[k] = event[k];
          }
      }
      return ret;
  }

  /*!
   * Core functionality for Snowplow Browser trackers v4.6.3 (http://bit.ly/sp-js)
   * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * Licensed under BSD-3-Clause
   */
  /**
   * Extract hostname from URL
   */
  function getHostName(url) {
      // scheme : // [username [: password] @] hostname [: port] [/ [path] [? query] [# fragment]]
      var e = new RegExp('^(?:(?:https?|ftp):)/*(?:[^@]+@)?([^:/#]+)'), matches = e.exec(url);
      return matches ? matches[1] : url;
  }
  /**
   * Cross-browser helper function to add event handler
   */
  function addEventListener(element, eventType, eventHandler, options) {
      if (element.addEventListener) {
          element.addEventListener(eventType, eventHandler, options);
          return true;
      }
      // IE Support
      if (element.attachEvent) {
          return element.attachEvent('on' + eventType, eventHandler);
      }
      element['on' + eventType] = eventHandler;
  }
  /**
   * Deletes an arbitrary cookie by setting the expiration date to the past
   *
   * @param cookieName - The name of the cookie to delete
   * @param domainName - The domain the cookie is in
   */
  function deleteCookie(cookieName, path, domainName, sameSite, secure) {
      cookie(cookieName, '', -1, path, domainName, sameSite, secure);
  }
  /**
   * Get and set the cookies associated with the current document in browser
   * This implementation always returns a string, returns the cookie value if only name is specified
   *
   * @param name - The cookie name (required)
   * @param value - The cookie value
   * @param ttl - The cookie Time To Live (seconds)
   * @param path - The cookies path
   * @param domain - The cookies domain
   * @param samesite - The cookies samesite attribute
   * @param secure - Boolean to specify if cookie should be secure
   * @returns string The cookies value
   */
  function cookie(name, value, ttl, path, domain, samesite, secure) {
      if (arguments.length > 1) {
          return (document.cookie =
              name +
                  '=' +
                  encodeURIComponent(value !== null && value !== void 0 ? value : '') +
                  (ttl ? '; Expires=' + new Date(+new Date() + ttl * 1000).toUTCString() : '') +
                  (path ? '; Path=' + path : '') +
                  (domain ? '; Domain=' + domain : '') +
                  (samesite ? '; SameSite=' + samesite : '') +
                  (secure ? '; Secure' : ''));
      }
      return decodeURIComponent((('; ' + document.cookie).split('; ' + name + '=')[1] || '').split(';')[0]);
  }
  /**
   * Convert a criterion object to a filter function
   *
   * @param object - criterion Either {allowlist: [array of allowable strings]}
   *                             or {denylist: [array of allowable strings]}
   *                             or {filter: function (elt) {return whether to track the element}
   * @param boolean - byClass Whether to allowlist/denylist based on an element's classes (for forms)
   *                        or name attribute (for fields)
   */
  function getFilterByClass(criterion) {
      // If the criterion argument is not an object, add listeners to all elements
      if (criterion == null || typeof criterion !== 'object' || Array.isArray(criterion)) {
          return function () {
              return true;
          };
      }
      var inclusive = Object.prototype.hasOwnProperty.call(criterion, 'allowlist');
      var specifiedClassesSet = getSpecifiedClassesSet(criterion);
      return getFilter(criterion, function (elt) {
          return checkClass(elt, specifiedClassesSet) === inclusive;
      });
  }
  /**
   * List the classes of a DOM element without using elt.classList (for compatibility with IE 9)
   */
  function getCssClasses(elt) {
      return elt.className.match(/\S+/g) || [];
  }
  /**
   * Check whether an element has at least one class from a given list
   */
  function checkClass(elt, classList) {
      var classes = getCssClasses(elt);
      for (var _i = 0, classes_1 = classes; _i < classes_1.length; _i++) {
          var className = classes_1[_i];
          if (classList[className]) {
              return true;
          }
      }
      return false;
  }
  function getFilter(criterion, fallbackFilter) {
      if (criterion.hasOwnProperty('filter') && criterion.filter) {
          return criterion.filter;
      }
      return fallbackFilter;
  }
  function getSpecifiedClassesSet(criterion) {
      // Convert the array of classes to an object of the form {class1: true, class2: true, ...}
      var specifiedClassesSet = {};
      var specifiedClasses = criterion.allowlist || criterion.denylist;
      if (specifiedClasses) {
          if (!Array.isArray(specifiedClasses)) {
              specifiedClasses = [specifiedClasses];
          }
          for (var i = 0; i < specifiedClasses.length; i++) {
              specifiedClassesSet[specifiedClasses[i]] = true;
          }
      }
      return specifiedClassesSet;
  }

  function newCookie(name) {
      var flushTimer;
      var lastSetValueArgs;
      var cacheExpireAt;
      var flushed = true;
      var flushTimeout = 10; // milliseconds
      var maxCacheTtl = 0.05; // seconds
      function getValue() {
          var _a;
          // Note: we can't cache the cookie value as we don't know the expiration date
          if (lastSetValueArgs && (!cacheExpireAt || cacheExpireAt > new Date())) {
              return (_a = lastSetValueArgs[0]) !== null && _a !== void 0 ? _a : cookie(name);
          }
          return cookie(name);
      }
      function setValue(value, ttl, path, domain, samesite, secure) {
          lastSetValueArgs = [value, ttl, path, domain, samesite, secure];
          flushed = false;
          // throttle setting the cookie
          if (flushTimer === undefined) {
              flushTimer = setTimeout(function () {
                  flushTimer = undefined;
                  flush();
              }, flushTimeout);
          }
          cacheExpireAt = new Date(Date.now() + Math.min(maxCacheTtl, ttl !== null && ttl !== void 0 ? ttl : maxCacheTtl) * 1000);
          return true;
      }
      function deleteValue(path, domainName, sameSite, secure) {
          lastSetValueArgs = undefined;
          flushed = true;
          // cancel setting the cookie
          if (flushTimer !== undefined) {
              clearTimeout(flushTimer);
              flushTimer = undefined;
          }
          deleteCookie(name, path, domainName, sameSite, secure);
      }
      function flush() {
          if (flushTimer !== undefined) {
              clearTimeout(flushTimer);
              flushTimer = undefined;
          }
          if (flushed) {
              return;
          }
          flushed = true;
          if (lastSetValueArgs !== undefined) {
              var value = lastSetValueArgs[0], ttl = lastSetValueArgs[1], path = lastSetValueArgs[2], domain = lastSetValueArgs[3], samesite = lastSetValueArgs[4], secure = lastSetValueArgs[5];
              cookie(name, value, ttl, path, domain, samesite, secure);
          }
      }
      return {
          getValue: getValue,
          setValue: setValue,
          deleteValue: deleteValue,
          flush: flush,
      };
  }
  /**
   * Create a new async cookie storage
   *
   * @returns A new cookie storage
   */
  function newCookieStorage() {
      var cache = {};
      function getOrInitCookie(name) {
          if (!cache[name]) {
              cache[name] = newCookie(name);
          }
          return cache[name];
      }
      function getCookie(name) {
          return getOrInitCookie(name).getValue();
      }
      function setCookie(name, value, ttl, path, domain, samesite, secure) {
          return getOrInitCookie(name).setValue(value, ttl, path, domain, samesite, secure);
      }
      function deleteCookie(name, path, domainName, sameSite, secure) {
          getOrInitCookie(name).deleteValue(path, domainName, sameSite, secure);
      }
      function clearCache() {
          cache = {};
      }
      function flush() {
          for (var _i = 0, _a = Object.values(cache); _i < _a.length; _i++) {
              var cookie_1 = _a[_i];
              cookie_1.flush();
          }
      }
      return {
          getCookie: getCookie,
          setCookie: setCookie,
          deleteCookie: deleteCookie,
          clearCache: clearCache,
          flush: flush,
      };
  }
  /**
   * Cookie storage instance with asynchronous cookie writes
   */
  var asyncCookieStorage = newCookieStorage();
  /**
   * Dispatch function to all specified trackers from the supplied collection
   *
   * @param trackers - An optional list of trackers to send the event to, or will send to all trackers
   * @param trackerCollection - The collection which the trackers will be selected from
   * @param fn - The function which will run against each tracker
   */
  function dispatchToTrackersInCollection(trackers, trackerCollection, fn) {
      try {
          getTrackersFromCollection(trackers !== null && trackers !== void 0 ? trackers : Object.keys(trackerCollection), trackerCollection).forEach(fn);
      }
      catch (ex) {
          LOG.error('Function failed', ex);
      }
  }
  function getTrackersFromCollection(trackerIds, trackerCollection) {
      var trackers = [];
      for (var _i = 0, trackerIds_1 = trackerIds; _i < trackerIds_1.length; _i++) {
          var id = trackerIds_1[_i];
          if (trackerCollection.hasOwnProperty(id)) {
              trackers.push(trackerCollection[id]);
          }
          else {
              LOG.warn(id + ' not configured');
          }
      }
      return trackers;
  }
  /**
   * Write all pending cookies to the browser.
   * Useful if you track events just before the page is unloaded.
   * This call is not necessary if `synchronousCookieWrite` is set to `true`.
   */
  function flushPendingCookies() {
      asyncCookieStorage.flush();
  }

  /*
   * Copyright (c) 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * All rights reserved.
   *
   * Redistribution and use in source and binary forms, with or without
   * modification, are permitted provided that the following conditions are met:
   *
   * 1. Redistributions of source code must retain the above copyright notice, this
   *    list of conditions and the following disclaimer.
   *
   * 2. Redistributions in binary form must reproduce the above copyright notice,
   *    this list of conditions and the following disclaimer in the documentation
   *    and/or other materials provided with the distribution.
   *
   * 3. Neither the name of the copyright holder nor the names of its
   *    contributors may be used to endorse or promote products derived from
   *    this software without specific prior written permission.
   *
   * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
   * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   */
  var TRACKABLE_ELEMENTS = ['a', 'area']; // should be kept in sync with TrackableElement type
  var TRACKABLE_ELEMENTS_SELECTOR = TRACKABLE_ELEMENTS.join(', ');
  var ELEMENT_PROTOCOL_FILTER = /^(javascript|vbscript|jscript|mocha|livescript|ecmascript):/i;
  var _trackers = {};
  var _listeners = {};
  var _configuration = {};
  var _logger = undefined;
  /**
   * Link click tracking.
   *
   * Will automatically track link clicks once enabled with `enableLinkClickTracking`
   * or you can manually track link clicks with `trackLinkClick`.
   *
   * @returns Plugin instance
   */
  function LinkClickTrackingPlugin() {
      return {
          activateBrowserPlugin: function (tracker) {
              _trackers[tracker.id] = tracker;
          },
          logger: function (logger) {
              _logger = logger;
          },
      };
  }
  /**
   * Enable link click tracking.
   *
   * @remarks
   * The default behaviour is to use actual click events. However, some browsers
   * (e.g., Firefox, Opera, and Konqueror) don't generate click events for the middle mouse button.
   *
   * To capture more "clicks", the pseudo click-handler uses mousedown + mouseup events.
   * This is not industry standard and is vulnerable to false positives (e.g., drag events).
   *
   * @param configuration The link tracking configuration to use for the new click handlers
   * @param trackers List of tracker IDs that should track the click events
   */
  function enableLinkClickTracking(configuration, trackers) {
      if (configuration === void 0) { configuration = {}; }
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      // remove listeners in case pseudoclick support has been toggled, which may duplicate handlers
      disableLinkClickTracking(trackers);
      trackers.forEach(function (id) {
          if (_trackers[id]) {
              configureLinkClickTracking(configuration, id);
              addClickListeners(id);
          }
      });
  }
  /**
   * Disable link click tracking.
   *
   * Removes all document-level click event handlers installed by the plugin for
   * provided tracker instances.
   *
   * @param trackers The tracker identifiers that will have their listeners removed
   */
  function disableLinkClickTracking(trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      trackers.forEach(function (id) {
          if (_trackers[id] && _listeners[id]) {
              // remove all possible cases where the handler may have been attached
              window.removeEventListener('click', _listeners[id], true);
              window.removeEventListener('mouseup', _listeners[id], true);
              window.removeEventListener('mousedown', _listeners[id], true);
          }
      });
  }
  /**
   * Add click event listeners to links which have been added to the page since the
   * last time `enableLinkClickTracking` or `refreshLinkClickTracking` was called.
   *
   * @deprecated v4.0 moved to event delegation and this is no longer required
   * @param trackers The tracker identifiers which the have their link click state refreshed
   */
  function refreshLinkClickTracking(_trackers) {
      _logger === null || _logger === void 0 ? void 0 : _logger.warn('refreshLinkClickTracking is deprecated in v4 and has no effect');
  }
  /**
   * Manually log a click.
   *
   * @param event The event or element information
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackLinkClick(event, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      dispatchToTrackersInCollection(trackers, _trackers, function (t) {
          var _a, _b, _c;
          var payload;
          if ('element' in event) {
              var includeContent = (_c = (_a = event.trackContent) !== null && _a !== void 0 ? _a : (_b = _configuration[t.id]) === null || _b === void 0 ? void 0 : _b.linkTrackingContent) !== null && _c !== void 0 ? _c : false;
              payload = processClick(event.element, includeContent);
          }
          else {
              payload = buildLinkClick(event);
          }
          if (payload)
              t.core.track(payload, event.context, event.timestamp);
      });
      flushPendingCookies();
  }
  /**
   * Process a clicked element into a link_click event payload.
   *
   * In case the href of the element is empty, "about:invalid" is used as the target URL.
   *
   * @param sourceElement The trackable element to be used to build the payload
   * @param includeContent Whether to include the element's contents in the payload
   * @returns A link_click SDE payload for the given element, or nothing if the element shouldn't be tracked
   */
  function processClick(sourceElement, includeContent) {
      if (includeContent === void 0) { includeContent = false; }
      var elementId, elementClasses, elementTarget, elementContent;
      var anchorElement = sourceElement;
      // browsers, such as Safari, don't downcase hostname and href
      var originalSourceHostName = anchorElement.hostname || getHostName(anchorElement.href);
      var targetUrl = anchorElement.href.replace(originalSourceHostName, function (s) { return s.toLowerCase(); });
      // Ignore script pseudo-protocol links
      if (!ELEMENT_PROTOCOL_FILTER.test(targetUrl)) {
          elementId = anchorElement.id;
          elementClasses = getCssClasses(anchorElement);
          elementTarget = anchorElement.target;
          elementContent = includeContent ? anchorElement.innerHTML : undefined;
          if (!targetUrl) {
              _logger === null || _logger === void 0 ? void 0 : _logger.warn('Link click target URL empty', anchorElement);
          }
          // decodeUrl %xx
          return buildLinkClick({
              targetUrl: targetUrl || 'about:invalid',
              elementId: elementId,
              elementClasses: elementClasses,
              elementTarget: elementTarget,
              elementContent: elementContent,
          });
      }
      return;
  }
  /**
   * Find the nearest trackable element to the given element; this is Element.closest()
   * with a polyfill if required.
   *
   * @param target
   * @returns An element that may be used to track a link click event, or null if `target` has no eligible parent
   */
  function findNearestEligibleElement(target) {
      if (target instanceof Element) {
          if (typeof target['closest'] === 'function')
              return target.closest(TRACKABLE_ELEMENTS_SELECTOR);
          var sourceElement = target;
          while (sourceElement) {
              var tagName = sourceElement.tagName.toLowerCase();
              if (TRACKABLE_ELEMENTS.indexOf(tagName) !== -1)
                  return sourceElement;
              sourceElement = sourceElement.parentElement;
          }
      }
      return null;
  }
  /**
   * Handle a (pseudo)click event; decide if the click is for a valid, unfiltered,
   * element and track the click.
   *
   * @param tracker Tracker ID to generate the event for
   * @param evt The DOM click event itself
   */
  function clickHandler(tracker, evt) {
      var context = _configuration[tracker].linkTrackingContext;
      var filter = _configuration[tracker].linkTrackingFilter;
      var event = evt || window.event;
      var button = event.which || event.button;
      var clicked = event.composed ? event.composedPath()[0] : event.target || event.srcElement;
      var target = findNearestEligibleElement(clicked);
      if (!target || target.href == null)
          return;
      if (filter && !filter(target))
          return;
      // Using evt.type (added in IE4), we avoid defining separate handlers for mouseup and mousedown.
      if (event.type === 'click') {
          trackLinkClick({
              element: target,
              context: resolveDynamicContext(context, target),
          }, [tracker]);
      }
      else if (event.type === 'mousedown') {
          if (button === 1 || button === 2) {
              _configuration[tracker].lastButton = button;
              _configuration[tracker].lastTarget = target;
          }
          else {
              delete _configuration[tracker].lastButton;
          }
      }
      else if (event.type === 'mouseup') {
          if (button === _configuration[tracker].lastButton && target === _configuration[tracker].lastTarget) {
              trackLinkClick({
                  element: target,
                  context: resolveDynamicContext(context, target),
              }, [tracker]);
          }
          delete _configuration[tracker].lastButton;
          delete _configuration[tracker].lastTarget;
      }
  }
  /**
   * Update the link-tracking configuration for the given tracker ID.
   *
   * @param param0 The new link-tracking configuration
   * @param tracker The tracker ID to update configuration for
   */
  function configureLinkClickTracking(_a, tracker) {
      var _b = _a === void 0 ? {} : _a, options = _b.options, pseudoClicks = _b.pseudoClicks, trackContent = _b.trackContent, context = _b.context;
      _configuration[tracker] = {
          linkTrackingContent: trackContent,
          linkTrackingContext: context,
          linkTrackingPseudoClicks: pseudoClicks,
          linkTrackingFilter: getFilterByClass(options),
      };
  }
  /**
   * Add (psuedo)click handlers to the window for the given tracker ID.
   *
   * @param trackerId Tracker ID to install a listener for
   */
  function addClickListeners(trackerId) {
      // by re-using exact function references the browser will prevent dupes and allow removal
      _listeners[trackerId] = _listeners[trackerId] || clickHandler.bind(null, trackerId);
      if (_configuration[trackerId].linkTrackingPseudoClicks) {
          // for simplicity and performance, we ignore drag events
          addEventListener(window, 'mouseup', _listeners[trackerId], true);
          addEventListener(window, 'mousedown', _listeners[trackerId], true);
      }
      else {
          addEventListener(window, 'click', _listeners[trackerId], true);
      }
  }

  exports.LinkClickTrackingPlugin = LinkClickTrackingPlugin;
  exports.disableLinkClickTracking = disableLinkClickTracking;
  exports.enableLinkClickTracking = enableLinkClickTracking;
  exports.refreshLinkClickTracking = refreshLinkClickTracking;
  exports.trackLinkClick = trackLinkClick;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=index.umd.js.map
